@@ -4,12 +4,13 @@
 # Tests for the `humanfriendly' package.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: September 10, 2015
+# Last Change: February 18, 2016
 # URL: https://humanfriendly.readthedocs.org
 
 """Test suite for the `humanfriendly` package."""
 
 # Standard library modules.
+import logging
 import math
 import os
 import random
@@ -19,8 +20,20 @@ import unittest
 
 # Modules included in our package.
 import humanfriendly
-import humanfriendly.cli
-from humanfriendly import compact, dedent
+from humanfriendly import cli, prompts
+from humanfriendly import compact, dedent, trim_empty_lines
+from humanfriendly.compat import StringIO
+from humanfriendly.prompts import (
+    TooManyInvalidReplies,
+    prompt_for_confirmation,
+    prompt_for_choice,
+    prompt_for_input,
+)
+from humanfriendly.sphinx import (
+    setup,
+    special_methods_callback,
+    usage_message_callback,
+)
 from humanfriendly.tables import (
     format_pretty_table,
     format_robust_table,
@@ -36,26 +49,33 @@ from humanfriendly.terminal import (
     ansi_wrap,
     connected_to_terminal,
     find_terminal_size,
+    terminal_supports_colors,
 )
 from humanfriendly.usage import (
     find_meta_variables,
     format_usage,
-    import_module,
     render_usage,
 )
 
-try:
-    # Python 2.x.
-    from StringIO import StringIO
-except ImportError:
-    # Python 3.x.
-    from io import StringIO
+# Test dependencies.
+from capturer import CaptureOutput
 
 
 class HumanFriendlyTestCase(unittest.TestCase):
 
     """Container for the `humanfriendly` test suite."""
 
+    def setUp(self):
+        """Configure logging to the terminal."""
+        try:
+            import coloredlogs
+            coloredlogs.install(level=logging.DEBUG)
+        except ImportError:
+            logging.basicConfig(
+                level=logging.DEBUG,
+                format='%(asctime)s %(name)s[%(process)d] %(levelname)s %(message)s',
+                datefmt='%Y-%m-%d %H:%M:%S')
+
     def test_compact(self):
         """Test :func:`humanfriendly.text.compact()`."""
         assert compact(' a \n\n b ') == 'a b'
@@ -408,35 +428,84 @@ class HumanFriendlyTestCase(unittest.TestCase):
             time.sleep(1)
 
     def test_prompt_for_choice(self):
-        """Test :func:`humanfriendly.prompt_for_choice`."""
-        interactive_prompt = humanfriendly.interactive_prompt
-        try:
-            # Choice selection by full string match.
-            humanfriendly.interactive_prompt = lambda prompt: 'foo'
-            self.assertEqual(humanfriendly.prompt_for_choice(['foo', 'bar']), 'foo')
-            # Choice selection by substring input.
-            humanfriendly.interactive_prompt = lambda prompt: 'f'
-            self.assertEqual(humanfriendly.prompt_for_choice(['foo', 'bar']), 'foo')
-            # Choice selection by number.
-            humanfriendly.interactive_prompt = lambda prompt: '2'
-            self.assertEqual(humanfriendly.prompt_for_choice(['foo', 'bar']), 'bar')
-            # Choice selection by going with the default.
-            humanfriendly.interactive_prompt = lambda prompt: ''
-            self.assertEqual(humanfriendly.prompt_for_choice(['foo', 'bar'], default='bar'), 'bar')
-            # Invalid substrings are refused.
-            responses = ['', 'q', 'z']
-            humanfriendly.interactive_prompt = lambda prompt: responses.pop(0)
-            self.assertEqual(humanfriendly.prompt_for_choice(['foo', 'bar', 'baz']), 'baz')
-            # Choice selection by substring input requires an unambiguous substring match.
-            responses = ['a', 'q']
-            humanfriendly.interactive_prompt = lambda prompt: responses.pop(0)
-            self.assertEqual(humanfriendly.prompt_for_choice(['foo', 'bar', 'baz', 'qux']), 'qux')
-            # Invalid numbers are refused.
-            responses = ['42', '2']
-            humanfriendly.interactive_prompt = lambda prompt: responses.pop(0)
-            self.assertEqual(humanfriendly.prompt_for_choice(['foo', 'bar', 'baz']), 'bar')
-        finally:
-            humanfriendly.interactive_prompt = interactive_prompt
+        """Test :func:`humanfriendly.prompts.prompt_for_choice()`."""
+        # Choice selection without any options should raise an exception.
+        self.assertRaises(ValueError, prompt_for_choice, [])
+        # If there's only one option no prompt should be rendered so we expect
+        # the following code to not raise an EOFError exception (despite
+        # connecting standard input to /dev/null).
+        with open(os.devnull) as handle:
+            with PatchedAttribute(sys, 'stdin', handle):
+                only_option = 'only one option (shortcut)'
+                assert prompt_for_choice([only_option]) == only_option
+        # Choice selection by full string match.
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: 'foo'):
+            assert prompt_for_choice(['foo', 'bar']) == 'foo'
+        # Choice selection by substring input.
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: 'f'):
+            assert prompt_for_choice(['foo', 'bar']) == 'foo'
+        # Choice selection by number.
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: '2'):
+            assert prompt_for_choice(['foo', 'bar']) == 'bar'
+        # Choice selection by going with the default.
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: ''):
+            assert prompt_for_choice(['foo', 'bar'], default='bar') == 'bar'
+        # Invalid substrings are refused.
+        replies = ['', 'q', 'z']
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: replies.pop(0)):
+            assert prompt_for_choice(['foo', 'bar', 'baz']) == 'baz'
+        # Choice selection by substring input requires an unambiguous substring match.
+        replies = ['a', 'q']
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: replies.pop(0)):
+            assert prompt_for_choice(['foo', 'bar', 'baz', 'qux']) == 'qux'
+        # Invalid numbers are refused.
+        replies = ['42', '2']
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: replies.pop(0)):
+            assert prompt_for_choice(['foo', 'bar', 'baz']) == 'bar'
+        # Test that interactive prompts eventually give up on invalid replies.
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: ''):
+            self.assertRaises(TooManyInvalidReplies, prompt_for_choice, ['a', 'b', 'c'])
+
+    def test_prompt_for_confirmation(self):
+        """Test :func:`humanfriendly.prompts.prompt_for_confirmation()`."""
+        # Test some (more or less) reasonable replies that indicate agreement.
+        for reply in 'yes', 'Yes', 'YES', 'y', 'Y':
+            with PatchedAttribute(prompts, 'interactive_prompt', lambda p: reply):
+                assert prompt_for_confirmation("Are you sure?") is True
+        # Test some (more or less) reasonable replies that indicate disagreement.
+        for reply in 'no', 'No', 'NO', 'n', 'N':
+            with PatchedAttribute(prompts, 'interactive_prompt', lambda p: reply):
+                assert prompt_for_confirmation("Are you sure?") is False
+        # Test that empty replies select the default choice.
+        for default_choice in True, False:
+            with PatchedAttribute(prompts, 'interactive_prompt', lambda p: ''):
+                assert prompt_for_confirmation("Are you sure?", default=default_choice) is default_choice
+        # Test that a warning is shown when no input nor a default is given.
+        replies = ['', 'y']
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: replies.pop(0)):
+            with CaptureOutput() as capturer:
+                assert prompt_for_confirmation("Are you sure?") is True
+                assert "there's no default choice" in capturer.get_text()
+        # Test that the default reply is shown in uppercase.
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: 'y'):
+            for default_value, expected_text in (True, 'Y/n'), (False, 'y/N'), (None, 'y/n'):
+                with CaptureOutput() as capturer:
+                    assert prompt_for_confirmation("Are you sure?", default=default_value) is True
+                    assert expected_text in capturer.get_text()
+        # Test that interactive prompts eventually give up on invalid replies.
+        with PatchedAttribute(prompts, 'interactive_prompt', lambda p: ''):
+            self.assertRaises(TooManyInvalidReplies, prompt_for_confirmation, "Are you sure?")
+
+    def test_prompt_for_input(self):
+        """Test :func:`humanfriendly.prompts.prompt_for_input()`."""
+        with open(os.devnull) as handle:
+            with PatchedAttribute(sys, 'stdin', handle):
+                # If standard input isn't connected to a terminal the default value should be returned.
+                default_value = "To seek the holy grail!"
+                assert prompt_for_input("What is your quest?", default=default_value) == default_value
+                # If standard input isn't connected to a terminal and no default value
+                # is given the EOFError exception should be propagated to the caller.
+                self.assertRaises(EOFError, prompt_for_input, "What is your favorite color?")
 
     def test_cli(self):
         """Test the command line interface."""
@@ -554,17 +623,26 @@ class HumanFriendlyTestCase(unittest.TestCase):
 
     def test_connected_to_terminal(self):
         """Test :func:`humanfriendly.terminal.connected_to_terminal()`."""
-        for stream in [sys.stdin, sys.stdout, sys.stderr]:
-            result = connected_to_terminal(stream)
-            # We really can't assert a True or False value here because this
-            # test suite should be able to run both interactively and
-            # non-interactively :-).
-            assert isinstance(result, bool)
-        # We can at least verify that e.g. /dev/null is never a terminal :-).
+        self.check_terminal_capabilities(connected_to_terminal)
+
+    def test_terminal_supports_colors(self):
+        """Test :func:`humanfriendly.terminal.terminal_supports_colors()`."""
+        self.check_terminal_capabilities(terminal_supports_colors)
+
+    def check_terminal_capabilities(self, test_stream):
+        """Helper for :func:`test_connected_to_terminal()` and func:`test_terminal_supports_colors()`."""
+        # This test suite should be able to run interactively as well as
+        # non-interactively, so we can't expect or demand that standard streams
+        # will always be connected to a terminal. Fortunately Capturer enables
+        # us to fake it :-).
+        for stream in sys.stdout, sys.stderr:
+            with CaptureOutput():
+                assert test_stream(stream)
+        # Test something that we know can never be a terminal.
         with open(os.devnull) as handle:
-            assert not connected_to_terminal(handle)
-        # We can also verify that objects without isatty() don't raise an exception.
-        assert not connected_to_terminal(object())
+            assert not test_stream(handle)
+        # Verify that objects without isatty() don't raise an exception.
+        assert not test_stream(object())
 
     def test_find_meta_variables(self):
         """Test :func:`humanfriendly.usage.find_meta_variables()`."""
@@ -619,12 +697,66 @@ class HumanFriendlyTestCase(unittest.TestCase):
                 Don't change anything.
         """) for token in ('`-n`', '`--dry-run`'))
 
-    def test_import_module(self):
-        """Test :func:`humanfriendly.usage.import_module()`."""
-        import humanfriendly
-        assert humanfriendly is import_module('humanfriendly')
-        from humanfriendly import cli
-        assert cli is import_module('humanfriendly.cli')
+    def test_sphinx_customizations(self):
+        """Test the :mod:`humanfriendly.sphinx` module."""
+        class FakeApp(object):
+
+            def __init__(self):
+                self.callbacks = {}
+
+            def __documented_special_method__(self):
+                """Documented unofficial special method."""
+                pass
+
+            def __undocumented_special_method__(self):
+                # Intentionally not documented :-).
+                pass
+
+            def connect(self, event, callback):
+                self.callbacks.setdefault(event, []).append(callback)
+
+            def bogus_usage(self):
+                """Usage: This is not supposed to be reformatted!"""
+                pass
+
+        # Test event callback registration.
+        fake_app = FakeApp()
+        setup(fake_app)
+        assert special_methods_callback in fake_app.callbacks['autodoc-skip-member']
+        assert usage_message_callback in fake_app.callbacks['autodoc-process-docstring']
+        # Test that `special methods' which are documented aren't skipped.
+        assert special_methods_callback(
+            app=None, what=None, name=None,
+            obj=FakeApp.__documented_special_method__,
+            skip=True, options=None,
+        ) is False
+        # Test that `special methods' which are undocumented are skipped.
+        assert special_methods_callback(
+            app=None, what=None, name=None,
+            obj=FakeApp.__undocumented_special_method__,
+            skip=True, options=None,
+        ) is True
+        # Test formatting of usage messages. obj/lines
+        from humanfriendly import cli, sphinx
+        # We expect the docstring in the `cli' module to be reformatted
+        # (because it contains a usage message in the expected format).
+        assert self.docstring_is_reformatted(cli)
+        # We don't expect the docstring in the `sphinx' module to be
+        # reformatted (because it doesn't contain a usage message).
+        assert not self.docstring_is_reformatted(sphinx)
+        # We don't expect the docstring of the following *method* to be
+        # reformatted because only *module* docstrings should be reformatted.
+        assert not self.docstring_is_reformatted(fake_app.bogus_usage)
+
+    def docstring_is_reformatted(self, entity):
+        """Check whether :func:`.usage_message_callback()` reformats a module's docstring."""
+        lines = trim_empty_lines(entity.__doc__).splitlines()
+        saved_lines = list(lines)
+        usage_message_callback(
+            app=None, what=None, name=None,
+            obj=entity, options=None, lines=lines,
+        )
+        return lines != saved_lines
 
 
 def main(*args, **kw):
@@ -639,7 +771,7 @@ def main(*args, **kw):
         sys.argv = [sys.argv[0]] + list(args)
         sys.stdin = input_buffer
         sys.stdout = output_buffer
-        humanfriendly.cli.main()
+        cli.main()
     except SystemExit as e:
         returncode = e.code or 1
     finally:
@@ -659,5 +791,33 @@ def normalize_timestamp(value, ndigits=1):
     return '%.2f' % round(float(value), ndigits=ndigits)
 
 
+class PatchedAttribute(object):
+
+    """Context manager that temporary replaces an object attribute."""
+
+    def __init__(self, obj, name, value):
+        """
+        Initialize a :class:`PatchedAttribute` object.
+
+        :param obj: The object to patch.
+        :param name: An attribute name.
+        :param value: The value to set.
+        """
+        self.obj = obj
+        self.name = name
+        self.value = value
+        self.saved_value = None
+
+    def __enter__(self):
+        """Replace (patch) the attribute."""
+        self.saved_value = getattr(self.obj, self.name)
+        setattr(self.obj, self.name, self.value)
+        return self.value
+
+    def __exit__(self, exc_type=None, exc_value=None, traceback=None):
+        """Restore the attribute to its original value."""
+        setattr(self.obj, self.name, self.saved_value)
+
+
 if __name__ == '__main__':
     unittest.main()
